//
//  MasterViewController.swift
//  schedule
//
//  Created by Илья Лошкарёв on 21.03.2018.
//  Copyright © 2018 SFEDU. All rights reserved.
//

import UIKit

class MasterViewController: UITableViewController {

    @IBOutlet var offlineNotice: UIView!
    
    var dataTask: StubDataLoader?
    var schedule: Schedule?
    
    override func viewDidLoad() {
        super.viewDidLoad()

        setupReachability()
        let url = Bundle.main.url(forResource: "schedule", withExtension: "json")!
        print(url.description)
        dataTask = StubDataLoader(from: url)
    }

    override func viewWillAppear(_ animated: Bool) {
        clearsSelectionOnViewWillAppear = splitViewController!.isCollapsed
        super.viewWillAppear(animated)
    }
    
    // MARK: - Networking
    
    func loadData() {
        UIApplication.shared.isNetworkActivityIndicatorVisible = true
        print("loading...")
        
        dataTask?.perform(in: URLSession.shared, withCompletion: self.onDataLoaded(_:), withError: self.onLoadError(_:))
    }
    
    func onDataLoaded(_ data: Schedule) {
        print("loaded")
        
        schedule = data
        DispatchQueue.main.async {
            UIApplication.shared.isNetworkActivityIndicatorVisible = false
            self.tableView.reloadData()
        }
    }
    
    func onLoadError( _ error: DataLoaderError) {
        print(error)
        
        var title, message: String
        switch error {
        case .sessionError(let localizedDescription):
            title = "Local Error"
            message = localizedDescription
        case .responseError:
            title = "Server Error"
            message = "Unexpected response from server."
        case .dataError:
            title = "Data Error"
            message = "Unexpected data format."
        }
        
        let alert = UIAlertController(title: title, message: message, preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: nil))
        
        DispatchQueue.main.async {
            UIApplication.shared.isNetworkActivityIndicatorVisible = false
            self.present(alert, animated: true, completion: nil)
        }
    }
    
    // MARK: - Reachability
    
    func setupReachability() {
        let reachability = (UIApplication.shared.delegate as! AppDelegate).reachability
        NotificationCenter.default.addObserver(
            self,
            selector: #selector(self.reachabilityChanged(_:)),
            name: .reachabilityChanged,
            object: reachability
        )
    }
    
    deinit {
        let reachability = (UIApplication.shared.delegate as! AppDelegate).reachability
        NotificationCenter.default.removeObserver(
            self,
            name: .reachabilityChanged,
            object: reachability
        )
    }
    
    @objc func reachabilityChanged(_ note: Notification) {
        let reachability = note.object as! Reachability
        print("reachabilityChanged: \(reachability.connection)")
        
        if reachability.connection == .none {
            animateView(offlineNotice, withDuration: 0.5, popup: true)
        } else {
            animateView(offlineNotice, withDuration: 0.5, popup: false, hideOnComletion: true)
        }
        
        loadData()
    }
    
    func animateView(_ view: UIView!, withDuration duration: TimeInterval, popup: Bool, hideOnComletion: Bool = false) {
        let defaultHeight: CGFloat = 48
        
        var startFrame, endFrame: CGRect
        if popup {
            startFrame = CGRect(origin: offlineNotice.frame.origin,
                                size: CGSize(width: offlineNotice.frame.width, height: 0))
            endFrame = CGRect(origin: offlineNotice.frame.origin,
                              size: CGSize(width: offlineNotice.frame.width, height: defaultHeight))
        } else {
            startFrame = CGRect(origin: offlineNotice.frame.origin,
                                size: CGSize(width: offlineNotice.frame.width, height: defaultHeight))
            endFrame = CGRect(origin: offlineNotice.frame.origin,
                              size: CGSize(width: offlineNotice.frame.width, height: 0))
        }
        
        view.frame = startFrame
        tableView.tableHeaderView = view
        
        UIView.animate(withDuration: duration, animations: {
            self.tableView.tableHeaderView?.frame = endFrame
        }, completion: {
            finished in
            guard finished && hideOnComletion else {return}
            self.tableView.tableHeaderView = nil
        })
    }

    // MARK: - Segues

    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "showDetail"
        {
            if  let indexPath = tableView.indexPathForSelectedRow,
                let object = schedule?.curriculaItems(forLesson: indexPath.row, onDay: indexPath.section),
                let navController = segue.destination as? UINavigationController,
                let controller = navController.topViewController as? DetailViewController
            {
                controller.detailItem = object
                
                controller.navigationItem.leftBarButtonItem = splitViewController?.displayModeButtonItem
                controller.navigationItem.leftItemsSupplementBackButton = true
            }
        }
    }

    // MARK: - TableViewDataSource

    override func numberOfSections(in tableView: UITableView) -> Int {
        // Разбивка по дням
        return schedule?.numberOfWorkDays ?? 0
    }
    
    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        // Названия дней
        return Calendar.current.weekdaySymbols[(section + Calendar.current.firstWeekday) % 7]
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // Количество уроков в заданный день
        return schedule?.numberOfLessons(onDay: section) ?? 0
    }
    
    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        // Высота строки зависит от количества предметов в паре
        let count = schedule?.numberOfItems(forLesson: indexPath.row, onDay: indexPath.section) ?? 0
        return CGFloat(count) * 60
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: LessonCell.reuseId, for: indexPath) as! LessonCell
        
        cell.setup(for: schedule?.timeslot(forLesson: indexPath.row, onDay: indexPath.section),
                   with: schedule?.curriculaItems(forLesson: indexPath.row, onDay: indexPath.section))
        return cell
    }
}
